summaryrefslogtreecommitdiffstats
path: root/api
diff options
context:
space:
mode:
Diffstat (limited to 'api')
-rw-r--r--api/channel.go53
-rw-r--r--api/channel_test.go62
2 files changed, 113 insertions, 2 deletions
diff --git a/api/channel_test.go b/api/channel_test.go
index d4fb11bd8..d65aff66c 100644
--- a/api/channel_test.go
+++ b/api/channel_test.go
@@ -320,6 +320,27 @@ func TestGetChannel(t *testing.T) {
if _, err := Client.UpdateLastViewedAt(channel2.Id); err != nil {
t.Fatal(err)
}
+
+ if resp, err := Client.GetChannel(channel1.Id, ""); err != nil {
+ t.Fatal(err)
+ } else {
+ data := resp.Data.(*model.ChannelData)
+ if data.Channel.DisplayName != channel1.DisplayName {
+ t.Fatal("name didn't match")
+ }
+
+ // test etag caching
+ if cache_result, err := Client.GetChannel(channel1.Id, resp.Etag); err != nil {
+ t.Fatal(err)
+ } else if cache_result.Data.(*model.ChannelData) != nil {
+ t.Log(cache_result.Data)
+ t.Fatal("cache should be empty")
+ }
+ }
+
+ if _, err := Client.GetChannel("junk", ""); err == nil {
+ t.Fatal("should have failed - bad channel id")
+ }
}
func TestGetMoreChannel(t *testing.T) {
@@ -366,6 +387,47 @@ func TestGetMoreChannel(t *testing.T) {
}
}
+func TestGetChannelCounts(t *testing.T) {
+ Setup()
+
+ team := &model.Team{DisplayName: "Name", Name: "z-z-" + model.NewId() + "a", Email: "test@nowhere.com", Type: model.TEAM_OPEN}
+ team = Client.Must(Client.CreateTeam(team)).Data.(*model.Team)
+
+ user := &model.User{TeamId: team.Id, Email: model.NewId() + "corey@test.com", Nickname: "Corey Hulen", Password: "pwd"}
+ user = Client.Must(Client.CreateUser(user, "")).Data.(*model.User)
+ store.Must(Srv.Store.User().VerifyEmail(user.Id))
+
+ Client.LoginByEmail(team.Name, user.Email, "pwd")
+
+ channel1 := &model.Channel{DisplayName: "A Test API Name", Name: "a" + model.NewId() + "a", Type: model.CHANNEL_OPEN, TeamId: team.Id}
+ channel1 = Client.Must(Client.CreateChannel(channel1)).Data.(*model.Channel)
+
+ channel2 := &model.Channel{DisplayName: "B Test API Name", Name: "a" + model.NewId() + "a", Type: model.CHANNEL_OPEN, TeamId: team.Id}
+ channel2 = Client.Must(Client.CreateChannel(channel2)).Data.(*model.Channel)
+
+ if result, err := Client.GetChannelCounts(""); err != nil {
+ t.Fatal(err)
+ } else {
+ counts := result.Data.(*model.ChannelCounts)
+
+ if len(counts.Counts) != 4 {
+ t.Fatal("wrong number of channel counts")
+ }
+
+ if len(counts.UpdateTimes) != 4 {
+ t.Fatal("wrong number of channel update times")
+ }
+
+ if cache_result, err := Client.GetChannelCounts(result.Etag); err != nil {
+ t.Fatal(err)
+ } else if cache_result.Data.(*model.ChannelCounts) != nil {
+ t.Log(cache_result.Data)
+ t.Fatal("result data should be empty")
+ }
+ }
+
+}
+
func TestJoinChannel(t *testing.T) {
Setup()
diff --git a/api/channel.go b/api/channel.go
index 803274d32..151627623 100644
--- a/api/channel.go
+++ b/api/channel.go
@@ -18,11 +18,13 @@ func InitChannel(r *mux.Router) {
sr := r.PathPrefix("/channels").Subrouter()
sr.Handle("/", ApiUserRequiredActivity(getChannels, false)).Methods("GET")
sr.Handle("/more", ApiUserRequired(getMoreChannels)).Methods("GET")
+ sr.Handle("/counts", ApiUserRequiredActivity(getChannelCounts, false)).Methods("GET")
sr.Handle("/create", ApiUserRequired(createChannel)).Methods("POST")
sr.Handle("/create_direct", ApiUserRequired(createDirectChannel)).Methods("POST")
sr.Handle("/update", ApiUserRequired(updateChannel)).Methods("POST")
sr.Handle("/update_desc", ApiUserRequired(updateChannelDesc)).Methods("POST")
sr.Handle("/update_notify_level", ApiUserRequired(updateNotifyLevel)).Methods("POST")
+ sr.Handle("/{id:[A-Za-z0-9]+}/", ApiUserRequiredActivity(getChannel, false)).Methods("GET")
sr.Handle("/{id:[A-Za-z0-9]+}/extra_info", ApiUserRequired(getChannelExtraInfo)).Methods("GET")
sr.Handle("/{id:[A-Za-z0-9]+}/join", ApiUserRequired(joinChannel)).Methods("POST")
sr.Handle("/{id:[A-Za-z0-9]+}/leave", ApiUserRequired(leaveChannel)).Methods("POST")
@@ -275,7 +277,7 @@ func updateChannelDesc(c *Context, w http.ResponseWriter, r *http.Request) {
func getChannels(c *Context, w http.ResponseWriter, r *http.Request) {
// user is already in the newtork
if result := <-Srv.Store.Channel().GetChannels(c.Session.TeamId, c.Session.UserId); result.Err != nil {
if result.Err.Message == "No channels were found" {
@@ -300,7 +302,7 @@ func getChannels(c *Context, w http.ResponseWriter, r *http.Request) {
func getMoreChannels(c *Context, w http.ResponseWriter, r *http.Request) {
// user is already in the newtork
if result := <-Srv.Store.Channel().GetMoreChannels(c.Session.TeamId, c.Session.UserId); result.Err != nil {
c.Err = result.Err
@@ -314,6 +316,22 @@ func getMoreChannels(c *Context, w http.ResponseWriter, r *http.Request) {
}
}
+func getChannelCounts(c *Context, w http.ResponseWriter, r *http.Request) {
+
+ // user is already in the team
+
+ if result := <-Srv.Store.Channel().GetChannelCounts(c.Session.TeamId, c.Session.UserId); result.Err != nil {
+ c.Err = model.NewAppError("getChannelCounts", "Unable to get channel counts from the database", result.Err.Message)
+ return
+ } else if HandleEtag(result.Data.(*model.ChannelCounts).Etag(), w, r) {
+ return
+ } else {
+ data := result.Data.(*model.ChannelCounts)
+ w.Header().Set(model.HEADER_ETAG_SERVER, data.Etag())
+ w.Write([]byte(data.ToJson()))
+ }
+}
+
func joinChannel(c *Context, w http.ResponseWriter, r *http.Request) {
params := mux.Vars(r)
@@ -548,6 +566,37 @@ func updateLastViewedAt(c *Context, w http.ResponseWriter, r *http.Request) {
w.Write([]byte(model.MapToJson(result)))
}
+func getChannel(c *Context, w http.ResponseWriter, r *http.Request) {
+ params := mux.Vars(r)
+ id := params["id"]
+
+ //pchan := Srv.Store.Channel().CheckPermissionsTo(c.Session.TeamId, id, c.Session.UserId)
+ cchan := Srv.Store.Channel().Get(id)
+ cmchan := Srv.Store.Channel().GetMember(id, c.Session.UserId)
+
+ if cresult := <-cchan; cresult.Err != nil {
+ c.Err = cresult.Err
+ return
+ } else if cmresult := <-cmchan; cmresult.Err != nil {
+ c.Err = cmresult.Err
+ return
+ } else {
+ data := &model.ChannelData{}
+ data.Channel = cresult.Data.(*model.Channel)
+ member := cmresult.Data.(model.ChannelMember)
+ data.Member = &member
+
+ if HandleEtag(data.Etag(), w, r) {
+ return
+ } else {
+ w.Header().Set(model.HEADER_ETAG_SERVER, data.Etag())
+ w.Header().Set("Expires", "-1")
+ w.Write([]byte(data.ToJson()))
+ }
+ }
+
+}
+
func getChannelExtraInfo(c *Context, w http.ResponseWriter, r *http.Request) {
params := mux.Vars(r)
@@ -275,7 +277,7 @@ func updateChannelDesc(c *Context, w http.ResponseWriter, r *http.Request) {
func getChannels(c *Context, w http.ResponseWriter, r *http.Request) {
- // user is already in the newtork
+ // user is already in the team
if result := <-Srv.Store.Channel().GetChannels(c.Session.TeamId, c.Session.UserId); result.Err != nil {
if result.Err.Message == "No channels were found" {
@@ -300,7 +302,7 @@ func getChannels(c *Context, w http.ResponseWriter, r *http.Request) {
func getMoreChannels(c *Context, w http.ResponseWriter, r *http.Request) {
- // user is already in the newtork
+ // user is already in the team
if result := <-Srv.Store.Channel().GetMoreChannels(c.Session.TeamId, c.Session.UserId); result.Err != nil {
c.Err = result.Err